﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;
using System.ComponentModel;
using DXTweets.Model;
using DXTweets.Common;
using System.Net.Http;
using Windows.Devices.Geolocation;

namespace DXTweets
{
    public class TweetsMapModel : INotifyPropertyChanged
    {
        private List<TweetLocation> _tweets = new List<TweetLocation>();
        private List<TweetLocation> _alltweets = new List<TweetLocation>();
        private string _searchTerm = "";
        private string _last_id = "";
        private Geoposition thispos;

        public TweetsMapModel()
        {

            _searchTerm = "DevExpress";
            getTweets(_searchTerm);
            RaisePropertyChanged("tweets");
        }

        public int PinsCount
        {
            get { return _alltweets.Count(); }
        }

        public async Task<bool> getDevicePosition()
        {
            bool bReturn = false;
            Geolocator thisLocator = new Geolocator();
            thispos = await thisLocator.GetGeopositionAsync();
            if (thispos != null)
            {
                bReturn = true;
            }
            return bReturn;
        }

        public void getNewTweets()
        {
            getTweets(_searchTerm);
            RaisePropertyChanged("tweets");
        }

        public List<TweetLocation> tweets
        {
            get { return _alltweets; }
        }

        public string searchTerm
        {
            get { return _searchTerm; }
            set
            {
                if (value != _searchTerm)
                {
                    _last_id = "";
                    _alltweets = new List<TweetLocation>();
                    _tweets = new List<TweetLocation>();
                }
                _searchTerm = value;
                getTweets(_searchTerm);
                RaisePropertyChanged("tweets");
            }
        }


        public event PropertyChangedEventHandler PropertyChanged;

        private void RaisePropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        private string getTwitterID(string strLink)
        {
            int lastSlash = strLink.LastIndexOf("/");
            int firstSlash = strLink.LastIndexOf("/", lastSlash - 1);
            string strLinkChunk = strLink.Substring(firstSlash);
            string str_TwitterID = strLinkChunk.Substring(1, strLinkChunk.Length - 8);
            return str_TwitterID;
        }
        private async void getTweets(string _searchTerm)
        {
            bool b = await getDevicePosition();
            if (b)
            {
                geoCodeService.GeocodeRequest geocodeRequest = new geoCodeService.GeocodeRequest();
                geocodeRequest.Credentials = new geoCodeService.Credentials();
                geocodeRequest.Credentials.ApplicationId = "AmSNFwVzMvaqFlCYQx9RRUfcAwSQCzi_Vcesric6JFQuBO9wZFXEsqzili-INaUA";
                //try
                //{
                    var httpClient = new HttpClient();
                    string posVal = thispos.Coordinate.Latitude.ToString() + "," + thispos.Coordinate.Longitude.ToString() + ",500mi";
                    string searchVal = "http://search.twitter.com/search.rss?rpp=20&geocode=" + posVal + "&q=" + _searchTerm;
                    var result = httpClient.GetStringAsync(searchVal);

                    var document = XDocument.Parse(result.Result);
                    var ns = (XNamespace)"http://base.google.com/ns/1.0";
                    if(document.Descendants("item").Count()>0)
                    {
                        string strLink = document.Descendants("item").First().Descendants("link").First().ToString();
                        string id = getTwitterID(strLink);
                        if (id != _last_id)
                        {
                            List<TweetLocation> tweetlocations = (from item in document.Descendants("item").Descendants(ns + "location")
                                                                  select new TweetLocation()
                                                                  {
                                                                      Address = item.Value
                                                                  }).ToList();
                            foreach (TweetLocation loc in tweetlocations)
                            {
                                if (loc.Address != "")
                                {
                                    geoCodeService.GeocodeServiceClient client = new geoCodeService.GeocodeServiceClient(new geoCodeService.GeocodeServiceClient.EndpointConfiguration());
                                    geocodeRequest.Query = loc.Address;
                                    geoCodeService.GeocodeResponse response = await client.GeocodeAsync(geocodeRequest);
                                    if (response.Results.Count() > 0)
                                    {
                                        geoCodeService.GeocodeResult thisresult = response.Results[0] as geoCodeService.GeocodeResult;
                                        if (thisresult.Locations.Count() > 0)
                                        {
                                            loc.Latitude = thisresult.Locations[0].Latitude;
                                            loc.Longitude = thisresult.Locations[0].Longitude;
                                        }
                                    }

                                }
                            }
                            _alltweets.InsertRange(0, tweetlocations);
                            _tweets = _alltweets.Take(100).ToList();
                            _last_id = id;
                            RaisePropertyChanged("tweets");
                        }
                    }
                //}
               // catch (Exception e)
                //{
                //    string strError = e.InnerException.Message;
               // }
            }
        }
    }
}
