﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Collections.ObjectModel;
using System.ComponentModel;
using DXTweets.Model;
using DXTweets.Common;
using System.Net.Http;
using System.Xml.Linq;

namespace DXTweets
{
    public class TweetsViewModel : INotifyPropertyChanged
    {
        private List<Tweet> _tweets = new List<Tweet>();
        private List<Tweet> _alltweets = new List<Tweet>();
        private List<Tweeter> _tweeters = new List<Tweeter>();

        private string _searchTerm = "";
        private string _last_id = "";
        public int total_tweet_count {get; set;}
        public int recent_tweet_count {get; set;}
        public int total_neutral_count {get; set;}
        public int total_positive_count {get; set;}
        public int total_negative_count {get; set;}
        public int recent_neutral_count {get; set;}
        public int recent_positive_count {get; set;}
        public int recent_negative_count {get; set;}
        private bool counts_updated=false;

        public TweetsViewModel()
        {
            _searchTerm = "DevExpress";
            getTweets(_searchTerm);
            
        }

        public void getNewTweets()
        {
            getTweets(_searchTerm);
            RaisePropertyChanged("tweets");
        }

        public List<Tweeter> tweeters
        {
            get { return _tweeters; }
            set { _tweeters = value; }
        }
        public List<Tweet> tweets
        {
            get { return _tweets; }
            set { _tweets = value; }
        }
        public string searchTerm
        {
            get { return _searchTerm; }
            set 
            {
                 if (value != _searchTerm)
                {
                    _last_id = "";
                    _alltweets = new List<Tweet>();
                    _tweets = new List<Tweet>();
                }
                _searchTerm = value;
                getTweets(_searchTerm);
                RaisePropertyChanged("tweets");
            }
        }

        

        public event PropertyChangedEventHandler PropertyChanged;

        private void RaisePropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        public async Task<bool> doTranslationCaller()
        {
            bool trans_done = await doTranslation();
            RaisePropertyChanged("tweets");
            return trans_done;
        }

        private async void getTweets(string _searchTerm)
        {
            try
            {
                var httpClient = new HttpClient();
                var result = httpClient.GetStringAsync("http://search.twitter.com/search.rss?rpp=20&q=" + _searchTerm);

                var document = XDocument.Parse(result.Result);
                var ns = (XNamespace)"http://base.google.com/ns/1.0";
                string strLink = document.Descendants("item").First().Descendants("link").First().ToString();
                string id = getTwitterID(strLink);
                if (id != _last_id)
                {
                    List<Tweet> tweets = (from item in document.Descendants("item")
                                          select new Tweet()
                                          {
                                              Title = (string)item.Descendants("title").First(),
                                              currentText = (string)item.Descendants("title").First(),
                                              Author = (string)item.Descendants("author").First(),
                                              pubDate = (string)item.Descendants("pubDate").First(),
                                              ImageUri = (string)item.Descendants(ns + "image_link").First()
                                          }).ToList();

                    _alltweets.InsertRange(0, tweets);
                    _tweets = _alltweets.Take(20).ToList();
                    
                    _last_id = id;
                    DoSentimentCount(tweets);
                    DoAuthorCount();
                }
            }
            catch(Exception e)
            {
            }
            RaisePropertyChanged("tweets");
        }

        private void DoAuthorCount()
        {
            _tweeters = new List<Tweeter>();
            List<string> authors = (from item in _alltweets
                                    select item.Author).Distinct().ToList();

            List<Tweeter> tweeterslist = (from item in _alltweets
                          group item by item.Author into grp
                          select new Tweeter
                          {
                              Author = grp.Key,
                              TweetCount = grp.Select(x=>x.Author).Count()
                          }).ToList();

            _tweeters = tweeterslist;
            RaisePropertyChanged("tweets");
        }

        private void DoSentimentCount(List<Tweet> newTweets)
        {
            total_tweet_count = _alltweets.Count();
            foreach(Tweet t in _alltweets)
            {
                string strText = t.currentText;
                if (strText.Contains(":)") || strText.Contains("cool") || strText.Contains("awesome") || strText.Contains("LOL") || strText.Contains("nice") || strText.Contains("love")
    || strText.Contains("good") || strText.Contains("fun") || strText.Contains("happy") || strText.Contains("sweet") || strText.Contains("hot"))
                {
                    total_positive_count++;
                }
                if (strText.Contains(":(") || strText.Contains("sucks") || strText.Contains("awful") || strText.Contains("stupid") || strText.Contains("bad") || strText.Contains("hate")
                    || strText.Contains("bad") || strText.Contains("idiot") || strText.Contains("blows") || strText.Contains("yuck") || strText.Contains("suck"))
                {
                    total_negative_count++;
                }

            }
            total_neutral_count = total_tweet_count - total_positive_count - total_negative_count;
            recent_tweet_count = newTweets.Count();
            foreach (Tweet t in newTweets)
            {
                string strText = t.currentText;
                if (strText.Contains(":)") || strText.Contains("cool") || strText.Contains("awesome") || strText.Contains("LOL") || strText.Contains("nice") || strText.Contains("love")
    || strText.Contains("good") || strText.Contains("fun") || strText.Contains("happy") || strText.Contains("sweet") || strText.Contains("hot"))
                {
                    recent_positive_count++;
                }
                if (strText.Contains(":(") || strText.Contains("sucks") || strText.Contains("awful") || strText.Contains("stupid") || strText.Contains("bad") || strText.Contains("hate")
                    || strText.Contains("bad") || strText.Contains("idiot") || strText.Contains("blows") || strText.Contains("yuck") || strText.Contains("suck"))
                {
                    recent_negative_count++;
                }
            }
            recent_neutral_count = recent_tweet_count - recent_positive_count - recent_negative_count;
            
        }
        private async Task<bool> doTranslation()
        {
            bool bReturn = true;
            string clientID = "LaurenceTest";
            string clientSecret = "ekRK0D0JYy7iy1oavWsWpAIQd795qh3Er/IrvqQWDXA=";
            var _Authentication = new AzureDataMarket(clientID, clientSecret);
            AzureDataMarket.Token m_Token = await _Authentication.GetTokenAsync();
            string auth = m_Token.Header;
            foreach (Tweet t in _tweets)
            {
                HttpClient client = new HttpClient();
                string strSource = t.currentText;
                client.DefaultRequestHeaders.Add("Authorization", auth);
                string RequestUri = "http://api.microsofttranslator.com/v2/Http.svc/Translate?text=" + System.Net.WebUtility.UrlEncode(strSource) + "&to=en";
                try
                {
                    string strTranslated = await client.GetStringAsync(RequestUri);
                    System.Xml.Linq.XDocument xTranslation = System.Xml.Linq.XDocument.Parse(strTranslated);
                    string strTransText = xTranslation.Root.FirstNode.ToString();
                    if (strTransText == strSource)
                        t.translatedText = "";
                    else
                        t.translatedText = strTransText;
                }
                catch (Exception ex)
                {
                    t.translatedText = t.currentText;
                    bReturn = false;
                }
            }
            
            return bReturn;
        }

        private string getTwitterID(string strLink)
        {
            int lastSlash = strLink.LastIndexOf("/");
            int firstSlash = strLink.LastIndexOf("/", lastSlash - 1);
            string strLinkChunk = strLink.Substring(firstSlash);
            string str_TwitterID = strLinkChunk.Substring(1, strLinkChunk.Length - 8);
            return str_TwitterID;
        }
    }
}
